"""
mars-MCD-helper package.

Utilities for retrieving and processing data from the Mars Climate Database
"""

from typing import List
from .read_mars_data import *
from .get_mars_data import *

__all__: List[str] = []  # noqa: WPS410 (the only __variable__ we use)


def get_parse_data(**kwargs):
    dataf = fetch_data(**kwargs)
    return read_ascii_data(dataf)



import requests
from bs4 import BeautifulSoup
from devtools import debug
from logging import getLogger, INFO, DEBUG
from pathlib import Path

logger = getLogger(__name__)


base_params = {
    "datekeyhtml": 1,
    "ls": 85.3,
    "localtime": 0.0,
    # "year": 2021,
    # "month": 8,
    # "day": 14,
    # "hours": 14,
    # "minutes": 26,
    # "seconds": 12,
    # "julian": "2459441.101527778",
    # "martianyear": 36,
    # "martianmonth": 3,
    # "sol": 183,
    "latitude": "all",
    "longitude": "all",
    "altitude": 10.0,
    "zkey": 3,
    "isfixedlt": "off",
    "dust": 1,
    "hrkey": 1,
    "zonmean": "off",
    "var1": "mtot",
    "var2": "t",
    "var3": "p",
    "var4": "none",
    "dpi": 80,
    "islog": "off",
    "colorm": "Blues",
    "minval": None,
    "maxval": None,
    "proj": "cyl",
    "plat": None,
    "plon": None,
    "trans": None,
    "iswind": "off",
    "latpoint": None,
    "lonpoint": None,
}


urlbase = "http://www-mars.lmd.jussieu.fr/mcd_python/"
url = urlbase + "cgi-bin/mcdcgi.py"


def generate_fn(**params):
    fn = "-".join(str(x) for _, x in params.items() if x)
    return f"marsdata_{fn}.txt"


def fetch_data(outdir=".", **params):
    p = base_params.copy()
    p.update(params)
    logger.info(f"Fetching page")
    r = requests.get(url, params=p)
    if "Ooops!" in r.text:
        raise Exception(f"Failed to download, server said {r.text}")
    soup = BeautifulSoup(r.text, features="html.parser")
    data_url = urlbase + soup.body.a["href"].replace("../", "")
    logger.info(f"Fetching ascii data from {data_url}")
    r = requests.get(data_url)
    fn = Path(outdir) / generate_fn(**params)
    with fn.open("w") as f:
        f.write(r.text)
    return f



"""
Entry-point module, in case you use `python -m mars_mcd_helper`.

Why does this file exist, and why `__main__`? For more info, read:

- https://www.python.org/dev/peps/pep-0338/
- https://docs.python.org/3/using/cmdline.html#cmdoption-m
"""

import sys

from mars_mcd_helper.cli import main

if __name__ == "__main__":
    sys.exit(main(sys.argv[1:]))



import csv
import pytest
from pathlib import Path
from typing import List, Union
from datetime import datetime
from collections import namedtuple
import numpy as np
import re


def parse_number(num: str) -> Union[float, int]:
    if num == "----":
        return None
    try:
        return int(num)
    except ValueError:
        return float(num)


def parse_header(lines: List[str]) -> dict:
    """Parse header."""
    # written to be readable by people beginning python, so rather verbose.
    data = {}
    match = re.search(r"MCD_(.+) with (.+).", lines[0])
    if match:
        data["mcd_version"] = match.group(1)
        data["model"] = match.group(2)
    match = re.search(r"Ls (.+). Altitude (.+) ALS Local time (.+)", lines[1])
    if match:
        data["ls"] = match.group(1)
        data["altitude"] = match.group(2)
        data["local_time"] = match.group(3).strip()
    assert "-" * 6 in lines[2]
    match = re.search(r"Column 1 is (.+)", lines[3])
    if match:
        data["column_1"] = match.group(1)

    match = re.search(r"Columns 2\+ are (.+)", lines[4])
    if match:
        data["variable"] = match.group(1)

    match = re.search(r"Line 1 is (.+)", lines[5])
    if match:
        data["keys"] = match.group(1)
    assert "-" * 6 in lines[6]
    match = re.search(r"Retrieved on: (.+)", lines[7])
    if match:
        data["retrieval_date"] = datetime.fromisoformat(match.group(1))
    return data


DataTable = namedtuple("DataTable", ["data", "xlabels", "ylabels"])


def parse_body(body: List[str]) -> "DataTable":
    # here we use the map (/reduce, but here we don't reduce) paradigm to show how sometimes functional programming is a *lot* simpler than writing the loops out by hand.
    # map applies a function (here an anonymous function decared with lambda) over an iterable
    # numpy has it's own map/reduce fns which are implemented in C and can be a lot faster than python's.
    body = map(lambda row: " ".join(row.strip().split()), body)
    body = list(body)
    xlabels = body[0].split("||")[1].strip().split(" ")
    body = body[2:]
    xlabels = map(lambda x: parse_number(x), xlabels)
    ylabels = map(lambda row: row.split("||")[0].strip(), body)
    ylabels = map(lambda x: parse_number(x), ylabels)
    data = map(lambda row: row.split("||")[1].strip().split(" "), body)
    data = np.array(list(data))
    data = DataTable(data, list(xlabels), list(ylabels))
    return data


def read_ascii_data(dataf):
    sections = {}
    with dataf.open() as f:
        row = f.readline()
        while True:
            if not row:
                break
            while "#" * 8 not in row:  # start header section
                row = f.readline()
            row = f.readline()  # skip ###### row
            header = []
            while "#" * 8 not in row:
                header.append(row)
                row = f.readline()
            header = parse_header(header)

            # parse body
            body = []
            row = f.readline()
            while row and "#" * 8 not in row:  # start header section
                body.append(row)
                row = f.readline()
            body = parse_body(body)
            header["data"] = body
            sections[header["variable"]] = header
    return sections


if __name__ == "__main__":
    inf = Path("data.txt")
    d = read_ascii_data(inf)
    print(d.keys())



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['mars_mcd_helper']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.9.3,<5.0.0',
 'numpy>=1.21.2,<2.0.0',
 'requests>=2.26.0,<3.0.0']

entry_points = \
{'console_scripts': ['mars-mcd-helper = mars_mcd_helper.cli:main']}

setup_kwargs = {
    'name': 'mars-mcd-helper',
    'version': '0.0.2',
    'description': 'Utilities for retrieving and processing data from the Mars Climate Database',
    'long_description': '# mars-MCD-helper\n\n[![ci](https://github.com/2e0byo/mars-mcd-helper/workflows/ci/badge.svg)](https://github.com/2e0byo/mars-mcd-helper/actions?query=workflow%3Aci)\n[![documentation](https://img.shields.io/badge/docs-mkdocs%20material-blue.svg?style=flat)](https://2e0byo.github.io/mars-mcd-helper/)\n[![pypi version](https://img.shields.io/pypi/v/mars-mcd-helper.svg)](https://pypi.org/project/mars-mcd-helper/)\n[![gitter](https://badges.gitter.im/join%20chat.svg)](https://gitter.im/mars-mcd-helper/community)\n\nUtilities for retrieving and processing data from the Mars Climate Database\n\n## Requirements\n\nmars-MCD-helper requires Python 3.7 or above.\n\n<details>\n<summary>To install Python 3.7, I recommend using <a href="https://github.com/pyenv/pyenv"><code>pyenv</code></a>.</summary>\n\n```bash\n# install pyenv\ngit clone https://github.com/pyenv/pyenv ~/.pyenv\n\n# setup pyenv (you should also put these three lines in .bashrc or similar)\nexport PATH="${HOME}/.pyenv/bin:${PATH}"\nexport PYENV_ROOT="${HOME}/.pyenv"\neval "$(pyenv init -)"\n\n# install Python 3.7\npyenv install 3.7.12\n\n# make it available globally\npyenv global system 3.7.12\n```\n</details>\n\n## Installation\n\nWith `pip`:\n```bash\npython3.7 -m pip install mars-mcd-helper\n```\n\nWith [`pipx`](https://github.com/pipxproject/pipx):\n```bash\npython3.7 -m pip install --user pipx\n\npipx install --python python3.7 mars-mcd-helper\n```\n',
    'author': 'John Morris',
    'author_email': 'john.morris@durham.ac.uk',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/2e0byo/mars-mcd-helper',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<3.11',
}


setup(**setup_kwargs)
